@@ -7,7 +7,7 @@ use core::num::NonZeroUsize;
 use alloc::rc::{Rc, Weak};
 
 use crate::tree::{
-    HierarchyEditGrantError, HierarchyEditProhibitionError, LockAggregatorForNode, TreeCore,
+    HierarchyEditGrantError, HierarchyEditProhibitionError, LockAggregatorForNode, Tree, TreeCore,
 };
 
 /// A membership of a node to a tree.
@@ -168,6 +168,13 @@ impl<T> Membership<T> {
         Rc::ptr_eq(&self.inner, &other.inner)
     }
 
+    /// Returns true if the node belong to the given tree.
+    #[inline]
+    #[must_use]
+    pub(crate) fn belongs_to(&self, tree: &Tree<T>) -> bool {
+        tree.ptr_eq_core(&*self.tree_core_ref())
+    }
+
     /// Returns true if the given node belong to the same tree.
     #[must_use]
     pub(crate) fn belongs_to_same_tree(&self, other: &Self) -> bool {
